/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2022 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include "listitemedit.h"

ListItemEdit::ListItemEdit(const QRegExp &rx, QObject *parent)
    : m_regExp(rx),
      QStyledItemDelegate(parent)
{

}

QWidget *ListItemEdit::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QLineEdit *editor = new QLineEdit(parent);
    editor->setValidator(new QRegExpValidator(m_regExp, parent));
    connect(editor, SIGNAL(textChanged(QString)), this, SIGNAL(textChanged(QString)));
    connect(editor, SIGNAL(editingFinished()), this, SIGNAL(editingFinished()));
    return editor;
}

void ListItemEdit::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    QLineEdit *lineEdit =  static_cast <QLineEdit*>(editor);
    QString text = index.model()->data(index, Qt::EditRole).toString();
    lineEdit->setText(text);
}

void ListItemEdit::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    QLineEdit *lineEdit =  static_cast <QLineEdit*>(editor);
    QString text = lineEdit->text();
    model->setData(index, text, Qt::EditRole);
}

void ListItemEdit::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    editor->setGeometry(option.rect);
}
